'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var postcss = _interopDefault(require('postcss'));
var lodash = require('lodash');
var cssToReactNative = require('css-to-react-native');
var pluginImports = _interopDefault(require('postcss-import'));
var pluginDefineProps = _interopDefault(require('postcss-define-property'));
var pluginNestedVars = _interopDefault(require('postcss-nested-vars'));
var pluginAtFor = _interopDefault(require('postcss-for'));
var pluginAtIf = _interopDefault(require('postcss-conditionals'));

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

const RULE_KEY_ATRULE = "atrule";
const RULE_KEY_RULE = "rule";
const RULE_KEY_PURESTYLE = "style-def";
const RULE_KEY_EXTEND = "extend";
const RULE_KEY_STYLE_PROPS = "decl";
const RULE_TOKEN_COMPONENT = "#";
const RULE_TOKEN_STYLE = ".";
const RULE_TOKEN_ASTERISK = "*";

const valueSanitizeRe = /^['"](.*)['"]$/;
function isStringValue(value) {
  return valueSanitizeRe.test(value);
}
function sanitizeValue(value) {
  if (!value) return "";
  return value.trim().replace(valueSanitizeRe, "$1");
}
const REGEX_IS_COMPONENT_STYLE = /((^[A-Z][^\s.]*)|^\*)(\.[a-z-A-Z]*)*$/;
/*

Text
*
*.line-through
View.flexible
Label.Capital-fffad
Slider-Container

*/

function isComponentStyle(selector) {
  return REGEX_IS_COMPONENT_STYLE.test(selector);
}

function getStyleKey(node) {
  let styleKey;
  const type = node.type,
        selector = node.selector,
        name = node.name,
        params = node.params;

  if (type === RULE_KEY_RULE) {
    const typeToken = selector.substr(0, 1);

    switch (typeToken) {
      case RULE_TOKEN_COMPONENT:
        styleKey = selector.substring(1);
        if (!isComponentStyle(styleKey)) styleKey = undefined;
        break;

      case RULE_TOKEN_ASTERISK:
      case RULE_TOKEN_STYLE:
        styleKey = selector;
        break;
    }
  } else if (type === RULE_KEY_ATRULE && name === RULE_KEY_PURESTYLE) {
    styleKey = params;
  }

  if (!styleKey) {
    throw new node.error(`Parser validation error. Invalid component style key \`${selector}\`.`, {
      word: selector
    });
  }

  return styleKey;
}
function getPathBreakdown(rule, root) {
  function _resolve(node, path) {
    const styleKey = getStyleKey(node);
    const hasParent = node.parent && node.parent != root;
    const paths = [styleKey, ...path];
    if (hasParent) return _resolve(node.parent, paths);
    return paths;
  }

  return _resolve(rule, []);
}

function getRuleMatcherByStyleKey(styleKey) {
  const typeToken = styleKey.substr(0, 1);

  switch (typeToken) {
    case RULE_TOKEN_STYLE:
    case RULE_TOKEN_COMPONENT:
      return {
        type: RULE_KEY_RULE,
        selector: styleKey
      };

    default:
      return {
        type: RULE_KEY_ATRULE,
        params: styleKey
      };
  }
}

function findClosestUpwards(root, node, styleKey) {
  const matched = walkForStyleKey(node, styleKey);

  if (!matched && node !== root) {
    // console.log('no matches at', node.params || node.selector)
    return findClosestUpwards(root, node.parent, styleKey);
  }

  return matched;
}

function walkForStyleKey(node, styleKey) {
  const matcher = getRuleMatcherByStyleKey(styleKey); // console.log('looking', matcher, 'at', node.params || node.selector, 'for', styleKey);

  if (node === undefined || node.nodes === undefined) return false;
  return lodash.find(node.nodes, matcher);
}

function findMatchingBySelector(origin, selector) {
  // console.log('findMatchingBySelector', selector);
  if (isStringValue(selector)) {
    const selectors = sanitizeValue(selector).split(" ");
    return lodash.reduce(selectors, (node, selector) => walkForStyleKey(node, selector), origin.root());
  } else {
    return findClosestUpwards(origin.root(), origin.parent.parent, selector);
  }
}

function atRules(root, rule) {
  //TODO: check if rule defined in right styleKey
  const isAllowedContainer = rule.parent.type == "rule";
  if (!isAllowedContainer) throw rule.error("@extend defined in wrong place.", {
    word: rule.name || rule.params
  });
  const match = findMatchingBySelector(rule, rule.params);
  if (!match) throw rule.error(`Could not found any matches for ${rule.params} to @extend style ${rule.parent.selector || rule.parent.params}.`, {
    word: rule.name || rule.params
  });
  lodash.each(match.nodes, decl => {
    rule.parent.append(decl.clone());
  });
  rule.remove();
}

var extendInternal = postcss.plugin("postcss-react-bender-extend", () => root => {
  root.walkAtRules(RULE_KEY_EXTEND, rule => atRules(root, rule));
});

/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @format
 * @flow
 */

function normalizeColor(color) {
  const matchers = getMatchers();
  let match;

  if (typeof color === "number") {
    if (color >>> 0 === color && color >= 0 && color <= 0xffffffff) {
      return color;
    }

    return null;
  } // Ordered based on occurrences on Facebook codebase


  if (match = matchers.hex6.exec(color)) {
    return parseInt(match[1] + "ff", 16) >>> 0;
  }

  if (names.hasOwnProperty(color)) {
    return names[color];
  }

  if (match = matchers.rgb.exec(color)) {
    return (// b
      (parse255(match[1]) << 24 | // r
      parse255(match[2]) << 16 | // g
      parse255(match[3]) << 8 | 0x000000ff) >>> // a
      0
    );
  }

  if (match = matchers.rgba.exec(color)) {
    return (// b
      (parse255(match[1]) << 24 | // r
      parse255(match[2]) << 16 | // g
      parse255(match[3]) << 8 | parse1(match[4])) >>> // a
      0
    );
  }

  if (match = matchers.hex3.exec(color)) {
    return parseInt(match[1] + match[1] + // r
    match[2] + match[2] + // g
    match[3] + match[3] + // b
    "ff", // a
    16) >>> 0;
  } // https://drafts.csswg.org/css-color-4/#hex-notation


  if (match = matchers.hex8.exec(color)) {
    return parseInt(match[1], 16) >>> 0;
  }

  if (match = matchers.hex4.exec(color)) {
    return parseInt(match[1] + match[1] + // r
    match[2] + match[2] + // g
    match[3] + match[3] + // b
    match[4] + match[4], // a
    16) >>> 0;
  }

  if (match = matchers.hsl.exec(color)) {
    return (hslToRgb(parse360(match[1]), // h
    parsePercentage(match[2]), // s
    parsePercentage(match[3]) // l
    ) | 0x000000ff) >>> // a
    0;
  }

  if (match = matchers.hsla.exec(color)) {
    return (hslToRgb(parse360(match[1]), // h
    parsePercentage(match[2]), // s
    parsePercentage(match[3]) // l
    ) | parse1(match[4])) >>> // a
    0;
  }

  return null;
}

function hue2rgb(p, q, t) {
  if (t < 0) {
    t += 1;
  }

  if (t > 1) {
    t -= 1;
  }

  if (t < 1 / 6) {
    return p + (q - p) * 6 * t;
  }

  if (t < 1 / 2) {
    return q;
  }

  if (t < 2 / 3) {
    return p + (q - p) * (2 / 3 - t) * 6;
  }

  return p;
}

function hslToRgb(h, s, l) {
  const q = l < 0.5 ? l * (1 + s) : l + s - l * s;
  const p = 2 * l - q;
  const r = hue2rgb(p, q, h + 1 / 3);
  const g = hue2rgb(p, q, h);
  const b = hue2rgb(p, q, h - 1 / 3);
  return Math.round(r * 255) << 24 | Math.round(g * 255) << 16 | Math.round(b * 255) << 8;
} // var INTEGER = '[-+]?\\d+';


const NUMBER = "[-+]?\\d*\\.?\\d+";
const PERCENTAGE = NUMBER + "%";

function call(...args) {
  return "\\(\\s*(" + args.join(")\\s*,\\s*(") + ")\\s*\\)";
}

let cachedMatchers;

function getMatchers() {
  if (cachedMatchers === undefined) {
    cachedMatchers = {
      rgb: new RegExp("rgb" + call(NUMBER, NUMBER, NUMBER)),
      rgba: new RegExp("rgba" + call(NUMBER, NUMBER, NUMBER, NUMBER)),
      hsl: new RegExp("hsl" + call(NUMBER, PERCENTAGE, PERCENTAGE)),
      hsla: new RegExp("hsla" + call(NUMBER, PERCENTAGE, PERCENTAGE, NUMBER)),
      hex3: /^#([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/,
      hex4: /^#([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/,
      hex6: /^#([0-9a-fA-F]{6})$/,
      hex8: /^#([0-9a-fA-F]{8})$/
    };
  }

  return cachedMatchers;
}

function parse255(str) {
  const int = parseInt(str, 10);

  if (int < 0) {
    return 0;
  }

  if (int > 255) {
    return 255;
  }

  return int;
}

function parse360(str) {
  const int = parseFloat(str);
  return (int % 360 + 360) % 360 / 360;
}

function parse1(str) {
  const num = parseFloat(str);

  if (num < 0) {
    return 0;
  }

  if (num > 1) {
    return 255;
  }

  return Math.round(num * 255);
}

function parsePercentage(str) {
  // parseFloat conveniently ignores the final %
  const int = parseFloat(str);

  if (int < 0) {
    return 0;
  }

  if (int > 100) {
    return 1;
  }

  return int / 100;
}

const names = {
  transparent: 0x00000000,
  // http://www.w3.org/TR/css3-color/#svg-color
  aliceblue: 0xf0f8ffff,
  antiquewhite: 0xfaebd7ff,
  aqua: 0x00ffffff,
  aquamarine: 0x7fffd4ff,
  azure: 0xf0ffffff,
  beige: 0xf5f5dcff,
  bisque: 0xffe4c4ff,
  black: 0x000000ff,
  blanchedalmond: 0xffebcdff,
  blue: 0x0000ffff,
  blueviolet: 0x8a2be2ff,
  brown: 0xa52a2aff,
  burlywood: 0xdeb887ff,
  burntsienna: 0xea7e5dff,
  cadetblue: 0x5f9ea0ff,
  chartreuse: 0x7fff00ff,
  chocolate: 0xd2691eff,
  coral: 0xff7f50ff,
  cornflowerblue: 0x6495edff,
  cornsilk: 0xfff8dcff,
  crimson: 0xdc143cff,
  cyan: 0x00ffffff,
  darkblue: 0x00008bff,
  darkcyan: 0x008b8bff,
  darkgoldenrod: 0xb8860bff,
  darkgray: 0xa9a9a9ff,
  darkgreen: 0x006400ff,
  darkgrey: 0xa9a9a9ff,
  darkkhaki: 0xbdb76bff,
  darkmagenta: 0x8b008bff,
  darkolivegreen: 0x556b2fff,
  darkorange: 0xff8c00ff,
  darkorchid: 0x9932ccff,
  darkred: 0x8b0000ff,
  darksalmon: 0xe9967aff,
  darkseagreen: 0x8fbc8fff,
  darkslateblue: 0x483d8bff,
  darkslategray: 0x2f4f4fff,
  darkslategrey: 0x2f4f4fff,
  darkturquoise: 0x00ced1ff,
  darkviolet: 0x9400d3ff,
  deeppink: 0xff1493ff,
  deepskyblue: 0x00bfffff,
  dimgray: 0x696969ff,
  dimgrey: 0x696969ff,
  dodgerblue: 0x1e90ffff,
  firebrick: 0xb22222ff,
  floralwhite: 0xfffaf0ff,
  forestgreen: 0x228b22ff,
  fuchsia: 0xff00ffff,
  gainsboro: 0xdcdcdcff,
  ghostwhite: 0xf8f8ffff,
  gold: 0xffd700ff,
  goldenrod: 0xdaa520ff,
  gray: 0x808080ff,
  green: 0x008000ff,
  greenyellow: 0xadff2fff,
  grey: 0x808080ff,
  honeydew: 0xf0fff0ff,
  hotpink: 0xff69b4ff,
  indianred: 0xcd5c5cff,
  indigo: 0x4b0082ff,
  ivory: 0xfffff0ff,
  khaki: 0xf0e68cff,
  lavender: 0xe6e6faff,
  lavenderblush: 0xfff0f5ff,
  lawngreen: 0x7cfc00ff,
  lemonchiffon: 0xfffacdff,
  lightblue: 0xadd8e6ff,
  lightcoral: 0xf08080ff,
  lightcyan: 0xe0ffffff,
  lightgoldenrodyellow: 0xfafad2ff,
  lightgray: 0xd3d3d3ff,
  lightgreen: 0x90ee90ff,
  lightgrey: 0xd3d3d3ff,
  lightpink: 0xffb6c1ff,
  lightsalmon: 0xffa07aff,
  lightseagreen: 0x20b2aaff,
  lightskyblue: 0x87cefaff,
  lightslategray: 0x778899ff,
  lightslategrey: 0x778899ff,
  lightsteelblue: 0xb0c4deff,
  lightyellow: 0xffffe0ff,
  lime: 0x00ff00ff,
  limegreen: 0x32cd32ff,
  linen: 0xfaf0e6ff,
  magenta: 0xff00ffff,
  maroon: 0x800000ff,
  mediumaquamarine: 0x66cdaaff,
  mediumblue: 0x0000cdff,
  mediumorchid: 0xba55d3ff,
  mediumpurple: 0x9370dbff,
  mediumseagreen: 0x3cb371ff,
  mediumslateblue: 0x7b68eeff,
  mediumspringgreen: 0x00fa9aff,
  mediumturquoise: 0x48d1ccff,
  mediumvioletred: 0xc71585ff,
  midnightblue: 0x191970ff,
  mintcream: 0xf5fffaff,
  mistyrose: 0xffe4e1ff,
  moccasin: 0xffe4b5ff,
  navajowhite: 0xffdeadff,
  navy: 0x000080ff,
  oldlace: 0xfdf5e6ff,
  olive: 0x808000ff,
  olivedrab: 0x6b8e23ff,
  orange: 0xffa500ff,
  orangered: 0xff4500ff,
  orchid: 0xda70d6ff,
  palegoldenrod: 0xeee8aaff,
  palegreen: 0x98fb98ff,
  paleturquoise: 0xafeeeeff,
  palevioletred: 0xdb7093ff,
  papayawhip: 0xffefd5ff,
  peachpuff: 0xffdab9ff,
  peru: 0xcd853fff,
  pink: 0xffc0cbff,
  plum: 0xdda0ddff,
  powderblue: 0xb0e0e6ff,
  purple: 0x800080ff,
  rebeccapurple: 0x663399ff,
  red: 0xff0000ff,
  rosybrown: 0xbc8f8fff,
  royalblue: 0x4169e1ff,
  saddlebrown: 0x8b4513ff,
  salmon: 0xfa8072ff,
  sandybrown: 0xf4a460ff,
  seagreen: 0x2e8b57ff,
  seashell: 0xfff5eeff,
  sienna: 0xa0522dff,
  silver: 0xc0c0c0ff,
  skyblue: 0x87ceebff,
  slateblue: 0x6a5acdff,
  slategray: 0x708090ff,
  slategrey: 0x708090ff,
  snow: 0xfffafaff,
  springgreen: 0x00ff7fff,
  steelblue: 0x4682b4ff,
  tan: 0xd2b48cff,
  teal: 0x008080ff,
  thistle: 0xd8bfd8ff,
  tomato: 0xff6347ff,
  turquoise: 0x40e0d0ff,
  violet: 0xee82eeff,
  wheat: 0xf5deb3ff,
  white: 0xffffffff,
  whitesmoke: 0xf5f5f5ff,
  yellow: 0xffff00ff,
  yellowgreen: 0x9acd32ff
};

/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @format
 */

const colorPropType = function colorPropType(isRequired, props, propName, componentName, location, propFullName) {
  const color = props[propName];

  if (color === undefined || color === null) {
    if (isRequired) {
      return new Error("Required " + location + " `" + (propFullName || propName) + "` was not specified in `" + componentName + "`.");
    }

    return;
  }

  if (typeof color === "number") {
    // Developers should not use a number, but we are using the prop type
    // both for user provided colors and for transformed ones. This isn't ideal
    // and should be fixed but will do for now...
    return;
  }

  if (normalizeColor(color) === null) {
    return new Error("Invalid " + location + " `" + (propFullName || propName) + "` supplied to `" + componentName + "`: " + color + "\n" + `Valid color formats are
  - '#f0f' (#rgb)
  - '#f0fc' (#rgba)
  - '#ff00ff' (#rrggbb)
  - '#ff00ff00' (#rrggbbaa)
  - 'rgb(255, 255, 255)'
  - 'rgba(255, 255, 255, 1.0)'
  - 'hsl(360, 100%, 100%)'
  - 'hsla(360, 100%, 100%, 1.0)'
  - 'transparent'
  - 'red'
  - 0xff00ff00 (0xrrggbbaa)
`);
  }
};

const ColorPropType = colorPropType.bind(null, false
/* isRequired */
);
ColorPropType.isRequired = colorPropType.bind(null, true
/* isRequired */
);

/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @flow
 * @format
 */

const ReactPropTypes = require("prop-types");
const ShadowPropTypesIOS = {
  shadowColor: ColorPropType,
  shadowOffset: ReactPropTypes.shape({
    width: ReactPropTypes.number,
    height: ReactPropTypes.number
  }),
  shadowOpacity: ReactPropTypes.number,
  shadowRadius: ReactPropTypes.number
};

/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @format
 * @flow
 */

const ReactPropTypes$1 = require("prop-types");

const TransformMatrixPropType = function TransformMatrixPropType(props, propName) {
  if (props[propName]) {
    return new Error("The transformMatrix style property is deprecated. " + "Use `transform: [{ matrix: ... }]` instead.");
  }
};

const DecomposedMatrixPropType = function DecomposedMatrixPropType(props, propName) {
  if (props[propName]) {
    return new Error("The decomposedMatrix style property is deprecated. " + "Use `transform: [...]` instead.");
  }
};

const DeprecatedTransformPropTypes = {
  transform: ReactPropTypes$1.arrayOf(ReactPropTypes$1.oneOfType([ReactPropTypes$1.shape({
    perspective: ReactPropTypes$1.number
  }), ReactPropTypes$1.shape({
    rotate: ReactPropTypes$1.string
  }), ReactPropTypes$1.shape({
    rotateX: ReactPropTypes$1.string
  }), ReactPropTypes$1.shape({
    rotateY: ReactPropTypes$1.string
  }), ReactPropTypes$1.shape({
    rotateZ: ReactPropTypes$1.string
  }), ReactPropTypes$1.shape({
    scale: ReactPropTypes$1.number
  }), ReactPropTypes$1.shape({
    scaleX: ReactPropTypes$1.number
  }), ReactPropTypes$1.shape({
    scaleY: ReactPropTypes$1.number
  }), ReactPropTypes$1.shape({
    translateX: ReactPropTypes$1.number
  }), ReactPropTypes$1.shape({
    translateY: ReactPropTypes$1.number
  }), ReactPropTypes$1.shape({
    skewX: ReactPropTypes$1.string
  }), ReactPropTypes$1.shape({
    skewY: ReactPropTypes$1.string
  })])),
  transformMatrix: TransformMatrixPropType,
  decomposedMatrix: DecomposedMatrixPropType
};

/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @format
 * @flow strict
 */

const ReactPropTypes$2 = require("prop-types");

const LayoutPropTypes = {
  display: ReactPropTypes$2.oneOf(["none", "flex"]),
  width: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  height: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  start: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  end: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  top: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  left: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  right: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  bottom: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  minWidth: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  maxWidth: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  minHeight: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  maxHeight: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  margin: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  marginVertical: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  marginHorizontal: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  marginTop: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  marginBottom: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  marginLeft: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  marginRight: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  marginStart: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  marginEnd: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  padding: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  paddingVertical: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  paddingHorizontal: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  paddingTop: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  paddingBottom: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  paddingLeft: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  paddingRight: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  paddingStart: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  paddingEnd: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  borderWidth: ReactPropTypes$2.number,
  borderTopWidth: ReactPropTypes$2.number,
  borderStartWidth: ReactPropTypes$2.number,
  borderEndWidth: ReactPropTypes$2.number,
  borderRightWidth: ReactPropTypes$2.number,
  borderBottomWidth: ReactPropTypes$2.number,
  borderLeftWidth: ReactPropTypes$2.number,
  position: ReactPropTypes$2.oneOf(["absolute", "relative"]),
  flexDirection: ReactPropTypes$2.oneOf(["row", "row-reverse", "column", "column-reverse"]),
  flexWrap: ReactPropTypes$2.oneOf(["wrap", "nowrap", "wrap-reverse"]),
  justifyContent: ReactPropTypes$2.oneOf(["flex-start", "flex-end", "center", "space-between", "space-around", "space-evenly"]),
  alignItems: ReactPropTypes$2.oneOf(["flex-start", "flex-end", "center", "stretch", "baseline"]),
  alignSelf: ReactPropTypes$2.oneOf(["auto", "flex-start", "flex-end", "center", "stretch", "baseline"]),
  alignContent: ReactPropTypes$2.oneOf(["flex-start", "flex-end", "center", "stretch", "space-between", "space-around"]),
  overflow: ReactPropTypes$2.oneOf(["visible", "hidden", "scroll"]),
  flex: ReactPropTypes$2.number,
  flexGrow: ReactPropTypes$2.number,
  flexShrink: ReactPropTypes$2.number,
  flexBasis: ReactPropTypes$2.oneOfType([ReactPropTypes$2.number, ReactPropTypes$2.string]),
  aspectRatio: ReactPropTypes$2.number,
  zIndex: ReactPropTypes$2.number,
  direction: ReactPropTypes$2.oneOf(["inherit", "ltr", "rtl"])
};

/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @flow
 * @format
 */

const ReactPropTypes$3 = require("prop-types");

const ImageStylePropTypes = _objectSpread({}, LayoutPropTypes, ShadowPropTypesIOS, DeprecatedTransformPropTypes, {
  resizeMode: ReactPropTypes$3.oneOf(["center", "contain", "cover", "repeat", "stretch"]),
  backfaceVisibility: ReactPropTypes$3.oneOf(["visible", "hidden"]),
  backgroundColor: ColorPropType,
  borderColor: ColorPropType,
  borderWidth: ReactPropTypes$3.number,
  borderRadius: ReactPropTypes$3.number,
  overflow: ReactPropTypes$3.oneOf(["visible", "hidden"]),

  /**
   * Changes the color of all the non-transparent pixels to the tintColor.
   */
  tintColor: ColorPropType,
  opacity: ReactPropTypes$3.number,

  /**
   * When the image has rounded corners, specifying an overlayColor will
   * cause the remaining space in the corners to be filled with a solid color.
   * This is useful in cases which are not supported by the Android
   * implementation of rounded corners:
   *   - Certain resize modes, such as 'contain'
   *   - Animated GIFs
   *
   * A typical way to use this prop is with images displayed on a solid
   * background and setting the `overlayColor` to the same color
   * as the background.
   *
   * For details of how this works under the hood, see
   * http://frescolib.org/docs/rounded-corners-and-circles.html
   *
   * @platform android
   */
  overlayColor: ReactPropTypes$3.string,
  // Android-Specific styles
  borderTopLeftRadius: ReactPropTypes$3.number,
  borderTopRightRadius: ReactPropTypes$3.number,
  borderBottomLeftRadius: ReactPropTypes$3.number,
  borderBottomRightRadius: ReactPropTypes$3.number
});

/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @format
 * @flow
 */

const ReactPropTypes$4 = require("prop-types");
/**
 * Warning: Some of these properties may not be supported in all releases.
 */

const ViewStylePropTypes = _objectSpread({}, LayoutPropTypes, ShadowPropTypesIOS, DeprecatedTransformPropTypes, {
  backfaceVisibility: ReactPropTypes$4.oneOf(["visible", "hidden"]),
  backgroundColor: ColorPropType,
  borderColor: ColorPropType,
  borderTopColor: ColorPropType,
  borderRightColor: ColorPropType,
  borderBottomColor: ColorPropType,
  borderLeftColor: ColorPropType,
  borderStartColor: ColorPropType,
  borderEndColor: ColorPropType,
  borderRadius: ReactPropTypes$4.number,
  borderTopLeftRadius: ReactPropTypes$4.number,
  borderTopRightRadius: ReactPropTypes$4.number,
  borderTopStartRadius: ReactPropTypes$4.number,
  borderTopEndRadius: ReactPropTypes$4.number,
  borderBottomLeftRadius: ReactPropTypes$4.number,
  borderBottomRightRadius: ReactPropTypes$4.number,
  borderBottomStartRadius: ReactPropTypes$4.number,
  borderBottomEndRadius: ReactPropTypes$4.number,
  borderStyle: ReactPropTypes$4.oneOf(["solid", "dotted", "dashed"]),
  borderWidth: ReactPropTypes$4.number,
  borderTopWidth: ReactPropTypes$4.number,
  borderRightWidth: ReactPropTypes$4.number,
  borderBottomWidth: ReactPropTypes$4.number,
  borderLeftWidth: ReactPropTypes$4.number,
  opacity: ReactPropTypes$4.number,

  /**
   * (Android-only) Sets the elevation of a view, using Android's underlying
   * [elevation API](https://developer.android.com/training/material/shadows-clipping.html#Elevation).
   * This adds a drop shadow to the item and affects z-order for overlapping views.
   * Only supported on Android 5.0+, has no effect on earlier versions.
   * @platform android
   */
  elevation: ReactPropTypes$4.number
});

/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @format
 * @flow
 */

const ReactPropTypes$5 = require("prop-types");

const TextStylePropTypes = _objectSpread({}, ViewStylePropTypes, {
  color: ColorPropType,
  fontFamily: ReactPropTypes$5.string,
  fontSize: ReactPropTypes$5.number,
  fontStyle: ReactPropTypes$5.oneOf(["normal", "italic"]),

  /**
   * Specifies font weight. The values 'normal' and 'bold' are supported for
   * most fonts. Not all fonts have a variant for each of the numeric values,
   * in that case the closest one is chosen.
   */
  fontWeight: ReactPropTypes$5.oneOf(["normal"
  /*default*/
  , "bold", "100", "200", "300", "400", "500", "600", "700", "800", "900"]),

  /**
   * @platform ios
   */
  fontVariant: ReactPropTypes$5.arrayOf(ReactPropTypes$5.oneOf(["small-caps", "oldstyle-nums", "lining-nums", "tabular-nums", "proportional-nums"])),
  textShadowOffset: ReactPropTypes$5.shape({
    width: ReactPropTypes$5.number,
    height: ReactPropTypes$5.number
  }),
  textShadowRadius: ReactPropTypes$5.number,
  textShadowColor: ColorPropType,

  /**
   * @platform ios
   */
  letterSpacing: ReactPropTypes$5.number,
  lineHeight: ReactPropTypes$5.number,

  /**
   * Specifies text alignment. The value 'justify' is only supported on iOS and
   * fallbacks to `left` on Android.
   */
  textAlign: ReactPropTypes$5.oneOf(["auto"
  /*default*/
  , "left", "right", "center", "justify"]),

  /**
   * @platform android
   */
  textAlignVertical: ReactPropTypes$5.oneOf(["auto"
  /*default*/
  , "top", "bottom", "center"]),

  /**
   * Set to `false` to remove extra font padding intended to make space for certain ascenders / descenders.
   * With some fonts, this padding can make text look slightly misaligned when centered vertically.
   * For best results also set `textAlignVertical` to `center`. Default is true.
   * @platform android
   */
  includeFontPadding: ReactPropTypes$5.bool,
  textDecorationLine: ReactPropTypes$5.oneOf(["none"
  /*default*/
  , "underline", "line-through", "underline line-through"]),

  /**
   * @platform ios
   */
  textDecorationStyle: ReactPropTypes$5.oneOf(["solid"
  /*default*/
  , "double", "dotted", "dashed"]),

  /**
   * @platform ios
   */
  textDecorationColor: ColorPropType,
  textTransform: ReactPropTypes$5.oneOf(["none"
  /*default*/
  , "capitalize", "uppercase", "lowercase"]),

  /**
   * @platform ios
   */
  writingDirection: ReactPropTypes$5.oneOf(["auto"
  /*default*/
  , "ltr", "rtl"])
});

const _require = require("lodash"),
      map = _require.map,
      filter = _require.filter,
      union = _require.union;

const ReactPropTypesSecret = require("prop-types/lib/ReactPropTypesSecret");
const StyleTypes = {
  STYLE_IMAGE: ":image",
  STYLE_TEXT: ":text",
  STYLE_VIEW: ":view"
};
const typesMap = {
  [StyleTypes.STYLE_IMAGE]: ImageStylePropTypes,
  [StyleTypes.STYLE_TEXT]: TextStylePropTypes,
  [StyleTypes.STYLE_VIEW]: ViewStylePropTypes
};
/**
 * Assert that the values match with the type specs.
 * Error messages are memorized and will only be shown once.
 *
 * @param {object} typeSpecs Map of name to a ReactPropType
 * @param {object} values Runtime values that need to be type-checked
 * @param {string} location e.g. "prop", "context", "child context"
 * @param {string} componentName Name of the component for error messages.
 * @private
 */

function checkPropTypes(typeSpecs, values, location, componentName) {
  for (var typeSpecName in typeSpecs) {
    if (typeSpecs.hasOwnProperty(typeSpecName)) {
      var error;

      try {
        // This is intentionally an invariant that gets caught. It's the same
        // behavior as without this statement except with a better message.
        if (typeof typeSpecs[typeSpecName] !== "function") {
          var err = Error((componentName || "React class") + ": " + location + " type `" + typeSpecName + "` is invalid; " + "it must be a function, usually from the `prop-types` package, but received `" + typeof typeSpecs[typeSpecName] + "`.");
          err.name = "Invariant Violation";
          throw err;
        }

        error = typeSpecs[typeSpecName](values, typeSpecName, componentName, location, null, ReactPropTypesSecret);
      } catch (ex) {
        error = ex;
      }

      if (error && !(error instanceof Error)) {
        error = new Error((componentName || "React class") + ": type specification of " + location + " `" + typeSpecName + "` is invalid; the type checker " + "function must return `null` or an `Error` but returned a " + typeof error + ". " + "You may have forgotten to pass an argument to the type checker " + "creator (arrayOf, instanceOf, objectOf, oneOf, oneOfType, and " + "shape all require an argument).");
      }

      if (error instanceof Error) {
        // Only monitor this failure once because there tends to be a lot of the
        // same error.
        // loggedTypeFailures[error.message] = true;
        error.typeSpecName = typeSpecName;
        error.componentName = componentName;
        error.location = location;
        error.values = values;
        return error; // printWarning(
        //   'Failed ' + location + ' type: ' + error.message
        // );
      }
    }
  }
}

function checkKeysAreExist(definitions, typeKeys) {
  const defKeys = Object.keys(definitions);
  const checked = filter(map(defKeys, key => {
    const check = typeKeys.indexOf(key) > -1;
    if (!check) return key;
  }), key => key !== undefined);
  return checked.length > 0 ? checked : undefined;
}

function validate(hash, definitions, optionalStrictType) {
  const checkAll = optionalStrictType == null;
  const checkTypes = checkAll ? [StyleTypes.STYLE_IMAGE, StyleTypes.STYLE_TEXT, StyleTypes.STYLE_VIEW] : [optionalStrictType];
  const typeKeys = [];
  const validated = map(checkTypes, type => {
    const types = typesMap[type];
    typeKeys.push(Object.keys(types));
    const violations = checkPropTypes(types, definitions, "style value", hash);
    return {
      type,
      violations
    };
  });
  const invalidKeys = checkKeysAreExist(definitions, union.apply(null, typeKeys));
  return {
    validated,
    invalidKeys
  };
}

function processStyleProps(opts, styleKey, declarations) {
  // if (declarations == null || declarations.length == 0)
  //   return undefined;
  const processedProps = lodash.reduce(declarations, (props, {
    prop,
    value
  }) => {
    const propertyName = cssToReactNative.getPropertyName(prop);
    const allowShorthand = opts.allowShorthand;
    const sanitised = sanitizeValue(value);
    const styles = cssToReactNative.getStylesForProperty(propertyName, sanitised, allowShorthand);
    return _objectSpread({}, props, styles);
  }, {});

  try {
    const _validate = validate(styleKey, processedProps),
          validated = _validate.validated,
          invalidKeys = _validate.invalidKeys;

    if (invalidKeys) {
      throw new Error(`Invalid style key(s) \`${invalidKeys.join(", ")}\` supplied to \`${styleKey}\``);
    } else {
      validated.forEach(({
        violations
      }) => {
        if (violations) throw violations;
      });
    }
  } catch (error) {
    if (error.typeSpecName) throw declarations[0].parent.error("Parser validation error. " + error.message, {
      word: error.typeSpecName
    });else throw declarations[0].error("Parser validation error. " + error.message, {
      word: error.typeSpecName
    });
  }

  return processedProps;
}

function setValueAtPath(setter, path, tree) {
  let leaf = tree;
  lodash.forEach(path, (key, id) => {
    const original = leaf[key] || {};
    if (id == path.length - 1) leaf[key] = setter(original);else leaf[key] = original;
    leaf = original;
  });
}

function processStyleRule(opts, styleRule, root) {
  if (styleRule == null || root == null || styleRule.nodes == null || styleRule.nodes.length == 0) return null;
  const styleKey = getStyleKey(styleRule);
  if (!styleKey) return null;
  const path = getPathBreakdown(styleRule, root);
  const value = processStyleProps(opts, styleKey, lodash.filter(styleRule.nodes, {
    type: RULE_KEY_STYLE_PROPS
  }));
  return {
    path,
    value
  };
}

function atRules$1(opts, {
  root,
  rule,
  tree
}) {
  if (rule.type === RULE_KEY_ATRULE) {
    if (rule.name === RULE_KEY_PURESTYLE) {
      const styleKey = getStyleKey(rule);

      if (styleKey) {
        const path = getPathBreakdown(rule, root);
        const styleValues = processStyleProps(opts, styleKey, lodash.filter(rule.nodes, {
          type: RULE_KEY_STYLE_PROPS
        })); // console.log('pure @ ', path, styleValues)

        setValueAtPath(original => {
          return _objectSpread({}, original, styleValues);
        }, path, tree);
      }
    }
  }
}
function rules(opts, {
  root,
  rule,
  tree
}) {
  // console.log('GG', rule.type, rule.selector)
  if (rule.type === RULE_KEY_RULE) {
    const processed = processStyleRule(opts, rule, root);

    if (processed) {
      const path = processed.path,
            value = processed.value; // const paths = processed.path;
      // getPathBreakdown(rule, root);
      // todo: fix
      // const paths = rule.selector.split(' ');
      // console.log('GG', rule.type, paths)

      setValueAtPath(original => {
        return _objectSpread({}, original, value);
      }, path, tree);
    }
  }
}

/**
 * opts: { allowShorthands: ?boolean, webpack: ?boolean }
 */

module.exports = postcss.plugin("postcss-react-bender", function (opts) {
  const pluginOptions = _objectSpread({
    allowShorthand: true,
    webpack: false
  }, opts);

  return postcss([pluginImports, pluginDefineProps, pluginNestedVars, pluginAtFor({
    resolve: pluginOptions.webpack ? require('./resolve-id') : undefined
  }), pluginAtIf, extendInternal]).use((root, result) => {
    let tree = {};
    root.walkAtRules(rule => atRules$1(pluginOptions, {
      root,
      rule,
      tree
    }));
    root.walkRules(rule => rules(pluginOptions, {
      root,
      rule,
      tree
    }));
    result.bender = tree;
  });
});
//# sourceMappingURL=index.cjs.js.map
